#include <stdint.h>
#include "pico/stdlib.h"
#include "pico/time.h"

#define SLEEP_MS 10

#include "Network_TTCan_Extract.h"

Network_TTCan_Impl_Controller_driver_input driver_input;

static Network_TTCan_Extract_modes_state modes_state;
static Network_TTCan_Extract_trigger_fetch_state fetch_state;
static Network_TTCan_Extract_controller_state controller_state;

static Network_TTCan_Types_error_severity pre_error = 0;
static Network_TTCan_Prim_Clocked_t__Network_TTCan_Types_ref_message pre_tx_ref;

void step1() {
  Network_TTCan_Impl_Controller_modes_result mode = Network_TTCan_Extract_modes_step(
    driver_input,
    pre_error,
    pre_tx_ref,
    &modes_state
  );
  // printf("%d %d %d\n", mode.cycle_time, mode.ref_trigger_offset, mode.sync_state);
  Network_TTCan_Impl_Triggers_fetch_result fetch = Network_TTCan_Extract_trigger_fetch_step(
    mode.ref_ck, mode.cycle_time, mode.cycle_index, mode.ref_trigger_offset,
    &fetch_state
  );
  Network_TTCan_Impl_Controller_controller_result control = Network_TTCan_Extract_controller_step(
    driver_input,
    mode.ref_ck, mode.mode, mode.cycle_index, mode.cycle_time, fetch, mode.sync_state, mode.error_CAN_Bus_Off, mode.error1,
    &controller_state
  );

  pre_error = control.error;
  pre_tx_ref = control.tx_ref;
}

#define ITERS 1

int main() {
    stdio_init_all();

    printf("Resetting...\n");

    Network_TTCan_Extract_modes_reset(&modes_state);
    Network_TTCan_Extract_trigger_fetch_reset(&fetch_state);
    Network_TTCan_Extract_controller_reset(&controller_state);

    while (1) {
      driver_input.mode_cmd.clock = rand();
      driver_input.mode_cmd.value = rand();
      driver_input.tx_status = rand();
      driver_input.bus_status = rand();
      driver_input.rx_ref.clock = rand();
      driver_input.rx_app.clock = rand();


      uint64_t start = to_us_since_boot(get_absolute_time());
      driver_input.local_time = start;
      for (uint32_t i = 0; i != ITERS; ++i)
        step1();
      uint64_t end = to_us_since_boot(get_absolute_time());
      uint64_t diff = end - start;
      printf(" %lld\n", (diff / ITERS));
      sleep_ms(10);
    }

    return 0;
}



const Network_TTCan_Types_trigger triggers[] = {
  { .trigger_type = Network_TTCan_Types_Tx_Trigger, .time_mark = 100, .cycle_offset = 0, .repeat_factor = 0, .message_index = 0},
  { .trigger_type = Network_TTCan_Types_Tx_Trigger, .time_mark = 200, .cycle_offset = 0, .repeat_factor = 1, .message_index = 1 },
  { .trigger_type = Network_TTCan_Types_Tx_Trigger, .time_mark = 200, .cycle_offset = 1, .repeat_factor = 1, .message_index = 2 },
  { .trigger_type = Network_TTCan_Types_Rx_Trigger, .time_mark = 300, .cycle_offset = 0, .repeat_factor = 0, .message_index = 3 },
  { .trigger_type = Network_TTCan_Types_Tx_Ref_Trigger, .time_mark = 1000, .cycle_offset = 0, .repeat_factor = 0 },
  { .trigger_type = Network_TTCan_Types_Watch_Trigger, .time_mark = 6000, .cycle_offset = 0, .repeat_factor = 0 },
};

static Network_TTCan_Types_trigger trigger_index(int32_t ix) {
  return triggers[ix];
}

const Network_TTCan_Types_config Network_TTCan_Extract_cfg = {
  .initial_ref_offset = 100,
  .master_index = { .tag = 0, .v = 0 },
  .tx_enable_window = 10,
  .cycle_count_max = 8,
  .severe_error_ref_cooldown = 65535,
  .trigger_index_fun = trigger_index,
  .expected_tx_triggers = 10
};

#include "Network_TTCan_Extract.c"